/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file gpio.c
*
* @author Freescale
*
* @version 0.0.1
*
* @date Jun 12, 2013
*
* @brief providing common gpio API. 
*
******************************************************************************/
#include "gpio.h"

/******************************************************************************
* Local variables
******************************************************************************/


/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local functions
*****************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

/******************************************************************************
* define GPIO APIs
*
*//*! @addtogroup gpio_api_list
* @{
*******************************************************************************/

/*****************************************************************************//*!
* @brief    Initialize the GPIO registers to the default reset values.
*        
* @param[in] pGPIO      Pointer to GPIO module, can be GPIOA/GPIOB.
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_DeInit(GPIO_Type *pGPIO)
{
    /* Sanity check */
    ASSERT((pGPIO == GPIOA) || (pGPIO == GPIOB));

    pGPIO->PCOR = 0x00000000;   /* Port Clear Output Register */
    pGPIO->PDDR = 0x00000000;   /* Port Data Direction */
    //pGPIO->PDIR = 0x00000000;   /* Port Data Input Register */
    pGPIO->PDOR = 0x00000000;   /* Port Data Output Register */
    pGPIO->PIDR = 0xFFFFFFFF;   /* Port Input Disable Register */
    pGPIO->PSOR = 0x00000000;   /* Port Set Output Register */
    pGPIO->PTOR = 0x00000000;   /* Port Toggle Output Register */
}

/*****************************************************************************//*!
* @brief    Initialize GPIO pins which are specified by u32PinMask
*        
* @param[in] pGPIO      Pointer to GPIO module, can be GPIOA/GPIOB.
* @param[in] u32PinMask GPIO pin mask need to be set
* @param[in] sGpioType  pin attribute 
*
* @return   none
*
* @Note
*   . High-current drive function is disabled, if the pin is configured as an input
*   . Internal pullup is disabled if the pin is configured as an output
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_Init(GPIO_Type *pGPIO, uint32_t u32PinMask, GPIO_PinConfigType sGpioType)
{
    /* Sanity check */
    ASSERT((pGPIO == GPIOA) || (pGPIO == GPIOB));
    
    if (sGpioType == GPIO_PinOutput)
    {
        pGPIO->PDDR |= u32PinMask;      /* Enable Port Data Direction Register */
        pGPIO->PIDR |= u32PinMask;      /* Set Port Input Disable Register */   
    }
    else if (sGpioType == GPIO_PinInput)
    {
        pGPIO->PDDR &= ~u32PinMask;   /* Disable Port Data Direction Register */
        pGPIO->PIDR &= ~u32PinMask;   /* Clear Port Input Disable Register */

        if (pGPIO == GPIOA)
        {
            PORT->PUEL &= ~u32PinMask;    /* Disable Pullup by defaut */
        }
        else
        {
            PORT->PUEH &= ~u32PinMask;    /* Disable Pullup by defaut */
        }
        
    }
    else if (sGpioType == GPIO_PinInput_InternalPullup)
    {
        pGPIO->PDDR &= ~u32PinMask;   /* Disable Port Data Direction Register */
        pGPIO->PIDR &= ~u32PinMask;   /* Clear Port Input Disable Register */

        if (pGPIO == GPIOA)
        {
            PORT->PUEL |= u32PinMask;
        }
        else
        {
            PORT->PUEH |= u32PinMask;
        }
        
    }
    else if (sGpioType == GPIO_PinOutput_HighCurrent)
    {
        pGPIO->PDDR |= u32PinMask;      /* Enable Port Data Direction Register */
        pGPIO->PIDR |= u32PinMask;      /* Set Port Input Disable Register */   

        /* Only PTH1/0, PTE1/0, PTD1/0, PTB5/4 support Hight-current Drive. */
        if (pGPIO == GPIOA)
        {
            if (u32PinMask & (1<<GPIO_PTD1))
            {   
                PORT->HDRVE |= PORT_HDRVE_PTD1_MASK;
            }
            if (u32PinMask & (1<<GPIO_PTD0))
            {
                PORT->HDRVE |= PORT_HDRVE_PTD0_MASK;
            }
            if (u32PinMask & (1<<GPIO_PTB5))
            {
                PORT->HDRVE |= PORT_HDRVE_PTB5_MASK;
            }            
            if (u32PinMask & (1<<GPIO_PTB4))
            {
                PORT->HDRVE |= PORT_HDRVE_PTB4_MASK;
            }
        }
        else
        {
            /* pGPIO == GPIOB */
            if (u32PinMask & GPIO_PTH1_MASK)
            {
                PORT->HDRVE |= PORT_HDRVE_PTH1_MASK;
            }
            if (u32PinMask & GPIO_PTH0_MASK)
            {
                PORT->HDRVE |= PORT_HDRVE_PTH0_MASK;
            }
            if (u32PinMask & GPIO_PTE1_MASK)
            {
                PORT->HDRVE |= PORT_HDRVE_PTE1_MASK;
            }            
            if (u32PinMask & GPIO_PTE0_MASK)
            {
                PORT->HDRVE |= PORT_HDRVE_PTE0_MASK;
            } 
        }
    }
    else
    {
        /* Default case */
    }
}

/*****************************************************************************//*!
* @brief    Toggle the pins which are specified by u32PinMask
*        
* @param[in] pGPIO       Pointer to GPIO module, can be GPIOA/GPIOB.
* @param[in] u32PinMask  Specify GPIO pin need to be toggled
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_Toggle(GPIO_Type *pGPIO, uint32_t u32PinMask)
{
    /* Sanity check */
    ASSERT((pGPIO == GPIOA) || (pGPIO == GPIOB));

    pGPIO->PTOR = u32PinMask;   /* Toggle the pins specified by u32PinMask */
}

/*****************************************************************************//*!
* @brief Read input data from GPIO which is specified by pGPIO
*        
* @param[in] pGPIO       Pointer to GPIO module, can be GPIOA/GPIOB.
*
* @return   GPIO input value unsigned int 32-bit
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
uint32_t GPIO_Read(GPIO_Type *pGPIO)
{
    /* Sanity check */
    ASSERT((pGPIO == GPIOA) || (pGPIO == GPIOB));

    return (pGPIO->PDIR);   /* Read Port Data Input Register */
  
}

/*****************************************************************************//*!
* @brief    Write output data to GPIO which is specified by pGPIO
*        
* @param[in] pGPIO       Pointer to GPIO module, can be GPIOA/GPIOB.
* @param[in] u32Value    value to output
*
* @return   none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_Write(GPIO_Type *pGPIO, uint32_t u32Value)
{
    /* Sanity check */
    ASSERT((pGPIO == GPIOA) || (pGPIO == GPIOB));
    
    pGPIO->PDOR = u32Value;    /* Write Port Ouput Data Register */
  
}

/*****************************************************************************//*!
* @brief    Initialize GPIO single pin which is specified by GPIO_Pin
*        
* @param[in] GPIO_Pin        GPIO pin name, can be GPIO_PTA0,1 ...
* @param[in] GPIO_PinConfig  Config output or input
*
* @return   none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_PinInit(GPIO_PinType GPIO_Pin, GPIO_PinConfigType GPIO_PinConfig)
{
    /* Sanity check */
    ASSERT(GPIO_Pin <= GPIO_PTI7);
    
    if (GPIO_Pin < GPIO_PTE0)
    {
        /* PTA0-7, PTB0-7, PTC0-7, PTD0-7 */
        if (GPIO_PinConfig == GPIO_PinOutput)
        {
            GPIOA->PDDR |= (1<<GPIO_Pin);      /* Enable Port Data Direction Register */
            GPIOA->PIDR |= (1<<GPIO_Pin);      /* Set Port Input Disable Register */
        }
        else if (GPIO_PinConfig == GPIO_PinInput)
        {
            GPIOA->PDDR &= ~(1<<GPIO_Pin);     /* Disable Port Data Direction Register */
            GPIOA->PIDR &= ~(1<<GPIO_Pin);     /* Clear Port Input Disable Register */

            PORT->PUEL &= ~(1<<GPIO_Pin);    /* Disable Pullup */
        }
        else if (GPIO_PinConfig == GPIO_PinInput_InternalPullup)
        {
            GPIOA->PDDR &= ~(1<<GPIO_Pin);     /* Disable Port Data Direction Register */
            GPIOA->PIDR &= ~(1<<GPIO_Pin);     /* Clear Port Input Disable Register */
            
            PORT->PUEL |= (1<<GPIO_Pin);    /* Enable Pullup */
        }
        else if (GPIO_PinConfig == GPIO_PinOutput_HighCurrent)
        {
            GPIOA->PDDR |= (1<<GPIO_Pin);      /* Enable Port Data Direction Register */
            GPIOA->PIDR |= (1<<GPIO_Pin);      /* Set Port Input Disable Register */   

            /* Only PTH1/0, PTE1/0, PTD1/0, PTB5/4 support Hight-current Drive. */
            if (GPIO_Pin == GPIO_PTD1)
            {   
                PORT->HDRVE |= PORT_HDRVE_PTD1_MASK;
            }
            else if (GPIO_Pin == GPIO_PTD0)
            {
                PORT->HDRVE |= PORT_HDRVE_PTD0_MASK;
            }
            else if (GPIO_Pin == GPIO_PTB5)
            {
                PORT->HDRVE |= PORT_HDRVE_PTB5_MASK;
            }            
            else if (GPIO_Pin == GPIO_PTB4)
            {
                PORT->HDRVE |= PORT_HDRVE_PTB4_MASK;
            }
            else
            {
                /* Default case */
            }
        }
        else
        {
            /* Default case */
        }
    }
    else
    {
        /* PTE0-7, PTF0-7, PTH0-7, PTI0-7 */
        GPIO_Pin = (GPIO_PinType)(GPIO_Pin - 32);

        if (GPIO_PinConfig == GPIO_PinOutput)
        {
            GPIOB->PDDR |= (1<<GPIO_Pin);      /* Enable Port Data Direction Register */
            GPIOB->PIDR |= (1<<GPIO_Pin);      /* Set Port Input Disable Register */
        }
        else if (GPIO_PinConfig == GPIO_PinInput)
        {
            GPIOB->PDDR &= ~(1<<GPIO_Pin);     /* Disable Port Data Direction Register */
            GPIOB->PIDR &= ~(1<<GPIO_Pin);     /* Clear Port Input Disable Register */

            PORT->PUEH &= ~(1<<GPIO_Pin);    /* Disable Pullup */
        }
        else if (GPIO_PinConfig == GPIO_PinInput_InternalPullup)
        {
            GPIOB->PDDR &= ~(1<<GPIO_Pin);     /* Disable Port Data Direction Register */
            GPIOB->PIDR &= ~(1<<GPIO_Pin);     /* Clear Port Input Disable Register */
            
            PORT->PUEH |= (1<<GPIO_Pin);    /* Enable Pullup */
        }
        else if (GPIO_PinConfig == GPIO_PinOutput_HighCurrent)
        {
            GPIOB->PDDR |= (1<<GPIO_Pin);      /* Enable Port Data Direction Register */
            GPIOB->PIDR |= (1<<GPIO_Pin);      /* Set Port Input Disable Register */   

            /* Only PTH1/0, PTE1/0, PTD1/0, PTB5/4 support Hight-current Drive. */
            if (GPIO_Pin == GPIO_PTH1)
            {   
                PORT->HDRVE |= PORT_HDRVE_PTH1_MASK;
            }
            else if (GPIO_Pin == GPIO_PTH0)
            {
                PORT->HDRVE |= PORT_HDRVE_PTH0_MASK;
            }
            else if (GPIO_Pin == GPIO_PTE5)
            {
                PORT->HDRVE |= PORT_HDRVE_PTE1_MASK;
            }            
            else if (GPIO_Pin == GPIO_PTE4)
            {
                PORT->HDRVE |= PORT_HDRVE_PTE0_MASK;
            }
            else
            {
                /* Default case */
            }
        }
        else
        {
            /* Default case */
        }
    }
}

/*****************************************************************************//*!
* @brief    Toggle GPIO single pin which is specified by GPIO_Pin
*        
* @param[in] GPIO_Pin        GPIO pin name, can be GPIO_PTA0,1 ...
*
* @return   none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_PinToggle(GPIO_PinType GPIO_Pin)
{
    /* Sanity check */
    ASSERT(GPIO_Pin <= GPIO_PTI7);

    if (GPIO_Pin < GPIO_PTE0)
    {
        /* PTA0-7, PTB0-7, PTC0-7, PTD0-7 */
        GPIOA->PTOR = (1<<GPIO_Pin);
    }
    else
    {
        /* PTE0-7, PTF0-7, PTH0-7, PTI0-7 */
        GPIO_Pin = (GPIO_PinType)(GPIO_Pin - GPIO_PTE0);
        GPIOB->PTOR = (1<<GPIO_Pin);
    }
}

/*****************************************************************************//*!
* @brief    Set GPIO single pin which is specified by GPIO_Pin
*        
* @param[in] GPIO_Pin        GPIO pin name, can be GPIO_PTA0,1 ...
*
* @return   none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_PinSet(GPIO_PinType GPIO_Pin)
{
    /* Sanity check */
    ASSERT(GPIO_Pin <= GPIO_PTI7);
    
    if (GPIO_Pin < GPIO_PTE0)
    {
        /* PTA0-7, PTB0-7, PTC0-7, PTD0-7 */
        GPIOA->PSOR = (1<<GPIO_Pin);
    }
    else
    {
        /* PTE0-7, PTF0-7, PTH0-7, PTI0-7 */
        GPIO_Pin = (GPIO_PinType)(GPIO_Pin - GPIO_PTE0);
        GPIOB->PSOR = (1<<GPIO_Pin);
    }
}

/*****************************************************************************//*!
* @brief    Clear GPIO single pin which is specified by GPIO_Pin
*        
* @param[in] GPIO_Pin        GPIO pin name, can be GPIO_PTA0,1 ...
*
* @return   none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/
void GPIO_PinClear(GPIO_PinType GPIO_Pin)
{
    /* Sanity check */
    ASSERT(GPIO_Pin <= GPIO_PTI7);
    
    if (GPIO_Pin < GPIO_PTE0)
    {
        /* PTA0-7, PTB0-7, PTC0-7, PTD0-7 */
        GPIOA->PCOR = (1<<GPIO_Pin);
    }
    else
    {
        /* PTE0-7, PTF0-7, PTH0-7, PTI0-7 */
        GPIO_Pin = (GPIO_PinType)(GPIO_Pin - 32);
        GPIOB->PCOR = (1<<GPIO_Pin);
    }
}

/*! @} End of gpio_api_list */

